package com.ivy.wallet.compose.helpers

import androidx.activity.ComponentActivity
import androidx.compose.ui.test.*
import androidx.compose.ui.test.junit4.AndroidComposeTestRule
import androidx.test.ext.junit.rules.ActivityScenarioRule

class BudgetModal<A : ComponentActivity>(
    private val composeTestRule: AndroidComposeTestRule<ActivityScenarioRule<A>, A>
) {
    private val amountInput = AmountInput(composeTestRule)

    fun enterName(budgetName: String) {
        composeTestRule.onNodeWithTag("base_input")
            .performTextReplacement(budgetName)
    }

    fun enterAmount(amount: String) {
        composeTestRule.onNodeWithTag("amount_balance")
            .performClick()

        amountInput.enterNumber(amount)
    }

    fun clickCategory(category: String) {
        composeTestRule.onNode(
            hasText(category)
                .and(hasAnyAncestor(hasTestTag("budget_categories_row"))),
            useUnmergedTree = true
        ).performClick()
    }

    fun clickAdd() {
        composeTestRule.onNodeWithText("Add")
            .performClick()
    }

    fun clickSave() {
        composeTestRule.onNodeWithText("Save")
            .performClick()
    }

    fun clickDelete() {
        composeTestRule.onNodeWithTag("modal_delete")
            .performClick()
    }

    fun clickClose() {
        composeTestRule.onNodeWithContentDescription("close")
            .performClick()
    }
}